#ifndef _AKIRA_LIB_H_
#define _AKIRA_LIB_H_
/***************************************************************************
 *  Utility functions C library
 *  akiralib.h
 *
 *  by Akira Tsukamoto
 ***************************************************************************/

#include "sysdep.h"

#define SECS_1900_1970_32 2208988800u   /* ntp:1900 gettimeofday:1970 */
#define SECS_1900_1970_16 1021084030u   /* diff of lower 16 bit of above, 
                                         * used for rtptime */

#ifdef  __cplusplus
extern "C" {
#endif

/*** global Variables ******************************************************/

/* extern char _aki_1024_[1024]; */

/** string to long. 
 * extended version of ANSI strtol() 
 * ANSI strtol() will return 0 on both error and string containing 0 value
 * but str2l() will return -1 on fail */
int str2l(long *result, const char *string, int radix, long min, long max);

/** how many times to retry for sendUDP() when sendto() ENOBUFS */
#define UDP_RETRY 3;

/** open UDP Port.
 * open UDP port with specified port number and returns fd */
int openUDPPort(in_port_t port);

/** WSAERRNO to errno */
int wsaetoe(int wsaerrno);

/** socket errno.
 * wrapper function for returning socket specific errno,
 * this function was made because win32's error returning
 * is different from unix   */
int sockerrno(void);

/** set sockaddr.
 * set internet socaddr(struct sockaddr_in) from IP and port number */
void setSockAddr(struct sockaddr_in *sAddr, struct in_addr *ip, in_port_t port);

/** get local hostname and ip number. */
in_addr_t getLHnameIp(char *hName, size_t len);

/** sockaddr to string.
 * convert sockaddr(struct sockaddr_in) to string
 * if host name was retrieved
 *   sockaddr -> www.suna-asobi.com(345.455.112.333):4123
 * if not
 *   sockaddr -> 345.455.112.333:4123  */
char *addr2str(char *string, size_t len, struct sockaddr_in *s);

/** check host.
 * check whether the sockAddr_in matches with ip and port
 * if port is 0, ignore port, check only ip
 * sockadd_in == (ip->s_addr, port)
 * match returns 0, false returns -1 */
int checkHost(struct sockaddr_in *sockAddr, struct in_addr *ip,
              in_port_t port);

/** get only filename from whole path */
void getFilenameFromPath(char *filename, size_t len, char *path);

/** sleep milli sec. */
void sleepMsec(unsigned long msec);

/** convert 64bit ntp format to timeval format */
struct timeval ntp2tv(struct timeval *ret,
                      uint32_t ntp_secs, uint32_t ntp_frac);

/** convert 32bit rtp time to timeval format */
struct timeval rtptime2tv(struct timeval *ret, uint32_t rtptime);

/** convert timeval format to 32bit rtp time */
uint32_t tv2rtptime(struct timeval *tv);

/** add timeval a + b */
struct timeval addTimeval(struct timeval *ret,
                          struct timeval *a, struct timeval *b);

/** sub timeval a - b */
struct timeval subTimeval(struct timeval *ret,
                          struct timeval *a, struct timeval *b);

/** max timeval.
 * select the timeval whichever is larger */
struct timeval maxTimeval(struct timeval *ret,
                          struct timeval *a, struct timeval *b);

/** tv to double.
 * convert struct_timval to double **/
double tv2double(struct timeval tv);

#ifdef  __cplusplus
}
#endif

#endif                          /* _AKIRA_LIB_H_ */
