#ifndef RTPQOS_H_
#define RTPQOS_H_
/***************************************************************************
 *
 *  RTP Quality of Service measurement
 *  rtpqos.h
 *
 *  Copyright 2002 by Columbia University; all rights reserved
 *  by Akira Tsukamoto
 *
 ***************************************************************************/

#include "rtp_api.h"

#define DEFAULT_PAYLOAD_TYPE 0      /* MULAW_8 */
#define DEFAULT_TTL          100
#define DEFAULT_PZT          20
#define DEFAULT_RTP_NAME     "rtpqos"
#define DEFAULT_SAMPLING     8000   /* This is used only when the rate was not 
                                     * specified in the IANA payload table */

#define DEFAULT_LOG   "rtpqos_log"

#define MAX_PAYLOAD_TYPE    127
#define MAX_RTP_DROPOUT     0x10    /* must be smaller than 0x7F */
#define MAX_PACK_INTERVAL   1000
#define MAX_RR_IN_COUMPOUND 100
#define MAX_PROVE_SEC       65535   /* longest probing time */

/** string buffer size, no less then 128 */
#define STRSIZE       1024

/** buffer for sendto() recvfrom() sockets. */
#define BUFSIZE       16384

/** Options struct.
 * contains all the settings from command line */
typedef struct options_s {
  char      argv0[NAME_MAX];          /** using it for error message */
  char      destHost[NAME_MAX];       /** destination host/port string */
  struct    in_addr destHostIpN;      /** destination host ip */
  in_port_t destHostPort;             /** destination host port */
  in_port_t localPort;                /** local binding port */
  char      logname[NAME_MAX];        /** filename for log */
  long      stop_tv_sec;              /** absolute ending timeval */
  long      probe_sec;                /** length of probing time */
  int       qos_mode;                 /** 0=match 1=listen */
  int       loopback_ok;              /** accept loopback packet */
  int       payloadType;              /** payload type of sending RTP data */
  int       samplingRate;             /** sampling rate of above payloadtype */
  int       packInterval;             /** packetizing interval */
  int       non_interactive;
  int       verbose;                  /** degree of verbose mode */
  FILE      *debfile;                 /** file descriptor for debug */
} Options;

/** RTP packet struct.
 * this will be descriptor RTP packets */
typedef struct rtp_s {
  context cid;
  int            rtp_fd;            /** RTP socket file descriptor */
  int            rtcp_fd;           /** RTCP socket file descriptor */
  FILE           *RTPlog;           /** logging file descriptor */
  FILE           *RTCPlog;          /** logging file descriptor */
  uint32_t       rtp_seq;
  uint32_t       rtcp_seq;
  struct timeval dept_tv;           /** departure time */
  struct timeval packInterval_tv;   /** RTP packet interval in struct timeval */
  int            datasize;          /** RTP data payload size */
} Rtp;

/** Statistics struct.
 * contains statisitcs information */
typedef struct statistics_s  {
  struct timeval  rtp_startime;     /** time of the first packet transmitted */
  struct timeval  rtp_lasttime;     /** time of the last packet transmitted */
  struct timeval  prev_arrive_tv;   /** previous arrival time, RTP packet */
  uint32_t        prev_ts;          /** previous time stamp,   RTP packet */
  int             recvRTPPack;      /** counter of received RTP packets */
  unsigned long   rtp_rttMax;       /** rtt  max     calced from RTP packets */
  unsigned long   rtp_rttAvr;       /** rtt  average calced from RTP packets */
  float           rtp_difMax;       /** diff max     calced from RTP packets */
  float           rtp_difAvr;       /** diff average calced from RTP packets */
  int             rtp_jitter;       /** jitter calculated from RTP packets */
  int             rtp_loss;         /** loss   calculated from RTP packets */
  struct timeval  rtcp_startime;    /** time, first received RTCP packet */
  struct timeval  rtcp_lasttime;    /** time, last  received RTCP packet */
  struct timeval  rtcp_sent_tv;     /** last sent time of RTCP SR packet */
  int             rtcp_sent_count;
  int             recvRTCPPack;     /** counter of received RTCP packets */
  unsigned long   rtcp_rttMax;      /** rtt max     calced from RTP packets */
  unsigned long   rtcp_rttAvr;      /** rtt average calced from RTP packets */
  Queue           *queue;
} Statistics;

typedef long unsigned  lu;      /* this is just for warning eater for print() */

#endif /* RTPQOS_H_ */
